function [ T1,Y1 ] = ABCDEFkin_2(varargin)
%ABCDEFkin Simulates kinetics of the reaction scheme below via numerical
%   integration of differential rate laws
%   A + B <-> C + D, D -> E, C <-> F
% Jason Kahn U Maryland 10/21/2016  version 1.1
% Arguments (all optional, but order is fixed and no skipping) in order are
%       A0,B0,C0,D0,E0,F0,k1,k2,k3,k4,k5,t_end,savefig
% Returns a vector of times and an array of []'s of A, B, C, D, E, F
% Makes two plots, one large to show all []'s and one small to evaluate SSA
%      and rapid equilibrium approximations for D and F
% Typical call with default rate constants and initial concentrations: 
%      [t,Y]= ABCDEFkin(1,.75,0,0,0,0,6,9,2,1,.1,5,0);
% This choice of k's demos a situation where the SSA applies to D and rapid eq between C and F
%      [t,Y]= ABCDEFkin(1,.75,0,0,0,0,6,9,200,1000,100,2,0);
% This shows a simple A + B -> C + D -> C + E and leaves off the savefig flag
%      [t,Y]= ABCDEFkin(1,1,0,0,0,0,6,0,2,0,0,2);
% Calls function ABCDEFdeltas (at the end of this file) to provide differential rates 
% Calls function export_fig if and only if savefig flag is set to 1
%   Download export_fig from http://www.mathworks.com/matlabcentral/fileexchange/23629-exportfig
%
% Define rate constants and initial concentrations
% Defaults:
A0 = 1; % M
B0 = 0.75;
C0 = 0;
D0 = 0;
E0 = 0;
F0 = 0;
k1 = 6; % M-1s-1
k2 = 9; % M-1s-1
k3 = 2; % s-1
k4 = 1; % M-1s-1
k5 = 0.1; % s-1
t_end = 5;
savefig = 0;  % Default is not to write files: savefig = 0
% Set savefig to 1 to activate saving images with exportfig
optargs = {A0,B0,C0,D0,E0,F0,k1,k2,k3,k4,k5,t_end,savefig};
numopts = 13;
% Change default values if user provides them
numvarargs = length(varargin);
if numvarargs > numopts
    error('ABCDEFkin:TooManyInputs', ...
        'ABCDEFkin allows at most %d optional inputs',numopts);
end

% now put the new values from varargin into the optargs cell array, 
% overwriting the default ones
optargs(1:numvarargs) = varargin;

% Place optional args back into into memorable variable names
[A0,B0,C0,D0,E0,F0,k1,k2,k3,k4,k5,t_end,savefig] = optargs{:};
concs0 = [A0,B0,C0,D0,E0,F0];
rateks = [k1,k2,k3,k4,k5];
% Generate strings for text boxes
str = {['k1 = ',num2str(k1),' M-1s-1'],['k2 = ',num2str(k2), ' M-1s-1'],...
    ['k3 = ',num2str(k3), ' s-1'],['k4 = ',num2str(k4), ' M-1s-1'],...
    ['k5 = ',num2str(k5), ' s-1'],['End time is ',num2str(t_end), ' s'],...
    ['Save figure flag is ',num2str(savefig)]};
str2 = {['A0 = ',num2str(A0),' M'],['B0 = ',num2str(B0), ' M'],...
    ['C0 = ',num2str(C0), ' M'],['D0 = ',num2str(D0), ' M'],...
    ['E0 = ',num2str(E0), ' M'],['F0 = ',num2str(F0), ' M']};
str3 = {['A + B <=> C + D, k1 and k2 '],['D -> E, k3'],['C + C <=> F, k4 and k5 ']};
% pnum is for numbering plots sequentially. 
% Limit number of files exported to lim_files
% savefig = 1 means we are saving files
if savefig == 1;
    lim_files = 50;
    fileout = 'ABCDEFkin';
    pnum = 1;
    while (pnum <= lim_files)
        fntest = [fileout,num2str(pnum),'.jpg'];
        if (exist(fntest,'file'))
            pnum = pnum+1;
        else
            disp(['Exported file is ',fntest])
            break
        end
    end
    if (pnum == lim_files+1)
        disp(['Exceeded file count: overwriting ',fntest])
    end
end

% Integrate differential equation to give time course. This is the
% point of the whole exercise. Integrate pre-steady state and long-time
% reaction separately. The ode15s solver is better for stiff differential
% equations.
options = odeset('RelTol',1e-6,'AbsTol',1e-4,'NonNegative',[1 2 3 4 5 6]);
[T1,Y1] = ode15s(@(t,concs) ABCDEFdeltas(t,concs,rateks),[0 t_end],concs0',options);
%
% First a large plain vanilla plot of the results:
%
figure1 = figure('Position',[10 10 1024 768]);
p1 = plot(T1,Y1,'LineWidth',2);
c1 = get(p1,'Color');
title(['ABCDEF Kinetics'],'FontSize',16)
%title(['ABCDEF kinetics: k values  ',num2str(rateks)],'FontSize',14)
xlabel('Time','FontSize',14)
ylabel('Concentrations','FontSize',14)
legend({'A','B','C','D','E','F'},'FontSize',14)
annotation(figure1,'textbox', [.2 .75 .135 .140],'BackGroundColor',[1 1 1],...
           'String', str2,'FitBoxToText','off','FontSize',14);
annotation(figure1,'textbox', [.6 .75 .135 .165],'BackGroundColor',[1 1 1],...
           'String', str,'FitBoxToText','off','FontSize',14);
annotation(figure1,'textbox', [.7 .55 .175 .130],'BackGroundColor',[1 1 1],...
           'String', str3,'FitBoxToText','on','FontSize',14);
%
% Second a small plot showing how well the SSA applies to D and the rapid equilibration
% assumption applies to F
%
figure2 = figure('Position',[50 50 512 384]);
hold on
% This plot illustrates approximations to the concentrations of D and F
% Plot actual [D]
plot(T1,Y1(:,4),'Linewidth',2,'Color',c1{4})
% SSA for [D]. d[D]/dt = k1*A*B - k2*C*D - k3*D = 0
ssad = k1.*Y1(:,1).*Y1(:,2)./(k2.*Y1(:,3) + k3);
plot(T1,ssad,'o','Color',c1{4})
% Actual F
plot(T1,Y1(:,6),'Linewidth',2,'Color',c1{6})
% Rapid eq approx for F: F = k4/k5*C^2
rapf = (k4/k5)*Y1(:,3).^2;
plot(T1,rapf,'x','Color',c1{6})
axis on
box on
axis([0 t_end 0 max([max(rapf),max(ssad),max(Y1(:,4)),max(Y1(:,6))])])
legend({'D','[D] SSA','F','F Rapid eq.'},'FontSize',14)
title(['ABCDEF Kinetics Approximations'],'FontSize',16)
%title(['ABCDEF kinetics: k values  ',num2str(rateks)],'FontSize',14)
xlabel('Time','FontSize',14)
ylabel('Concentrations','FontSize',14)
%
if (savefig == 1)
    export_fig (figure1,fntest)
end
end

function dconcs = ABCDEFdeltas(~,concs,rateks)
% Calculate the change in concentration of Efree, ES, S, and P
% return the change in each concentration
% These deltas are integrated in the calling program. 
% Assumes initial rate conditions -- no product inhibition
dconcs = zeros(6,1);
dconcs(1) = -rateks(1)*concs(1)*concs(2) + rateks(2)*concs(3)*concs(4);
dconcs(2) = dconcs(1);
dconcs(3) = rateks(1)*concs(1)*concs(2) - rateks(2)*concs(3)*concs(4)...
    - 2*rateks(4)*concs(3)^2 + 2*rateks(5)*concs(6);
dconcs(4) = rateks(1)*concs(1)*concs(2) - rateks(2)*concs(3)*concs(4) - rateks(3)*concs(4);
dconcs(5) = rateks(3)*concs(4);
dconcs(6) = rateks(4)*concs(3)^2 - rateks(5)*concs(6);
end

